/*
 * (c) 2008-2009 Technische Universität Dresden
 * This file is part of TUD:OS and distributed under the terms of the
 * GNU General Public License 2.
 * Please see the COPYING-GPL-2 file for details.
 */
#include "script_invoke.h"

#include <l4/re/service>
#include <l4/re/util/name_space_svr>
#include <l4/re/util/cap_alloc>
#include <l4/re/env>
#include "local_service.h"

#include <cstring>

#include "config_namespace.h"
#include "name_space.h"
#include "debug.h"


using namespace Ldr;

static
Ldr::Parser::Val
open_remote_service(Ldr::Scanner::Context const &ctxt, L4::Cap<void> svc,
                    int argc, cxx::String const *argv, bool recycle_cap)
{
  (void)argc;
  L4::Cap<L4::Kobject> new_cap;
  if (recycle_cap)
    new_cap = L4::cap_cast<L4::Kobject>(svc);
  else
    new_cap = L4Re::Util::cap_alloc.alloc<L4::Kobject>();

  int err = L4::cap_cast<L4Re::Service>(svc)->open(new_cap, argv[0].start(),
                                                   argv[0].len());
  if (err < 0)
    {
      Err().printf("%s:%d: invocation of service failed: '%s'\n",
	  ctxt.file(), ctxt.line(), l4sys_errtostr(err));
      L4Re::Util::cap_alloc.free(new_cap);
      throw Ldr::Parse_exception(ctxt, "");
    }

  return Ldr::Parser::Val(new_cap, L4Re::Util::Names::Obj::F_rw);
}

static
Ldr::Parser::Val
open_local_service(Ldr::Scanner::Context const &ctxt, L4::Server_object *svc,
                   int argc, cxx::String const *argv)
{
  Ldr::Local_service *lsvc = dynamic_cast<Ldr::Local_service *>(svc);
  if (!lsvc)
    throw Ldr::Parse_exception(ctxt, "invocation not supported");

  return Ldr::Parser::Val(lsvc->open(argc, argv));
}


Ldr::Parser::Val
Ldr_obj_mgr::invoke(Parser::Val const &obj, int argc,
                    cxx::String const *argv,
                    Scanner::Context const &ctxt) const
{
  switch (obj.type())
    {
    case Ldr::Parser::Val::Cap:
      // reuse the svc_cap for the new object got by open
      return open_remote_service(ctxt, obj.cap(), argc, argv, true);

    case Ldr::Parser::Val::Local_entry:
      if (!obj.entry()->obj()->is_trusted())
	throw Ldr::Parse_exception(ctxt, "cannot call untrusted object");

      if(obj.entry()->obj()->is_local())
	return open_local_service(ctxt, obj.entry()->obj()->obj(), argc, argv);
      else
	return open_remote_service(ctxt, obj.entry()->obj()->cap(), argc, argv, false);

    case Ldr::Parser::Val::Server_object:
      return open_local_service(ctxt, obj.svr_obj(), argc, argv);

    default:
      throw Ldr::Parse_exception(ctxt, "invocation not supported");
    }
}


Ldr::Parser::Val
Ldr_obj_mgr::find_object(cxx::String const &name,
                         Ldr::Scanner::Context const &ctxt) const
{
  if (name.starts_with("ns/"))
    {
      // look in a local name space
      Names::Entry *src = Ldr::local_root_ns()->find_iter(name.substr(3));
      if (!src)
	throw Ldr::Parse_exception(ctxt, Ldr::Token(Ldr::Token::Name, name),
	                           "object not found:");
      return src;
    }
  else
    {
      L4::Cap<void> cap = L4Re::Util::cap_alloc.alloc<void>();
      l4_umword_t local_id = 0;
      int err = L4Re::Env::env()->names()->query(name.start(), name.len(), cap, 0, &local_id);
      if (err < 0)
	{
	  L4Re::Util::cap_alloc.free(cap);
	  throw Ldr::Parse_exception(ctxt, Ldr::Token(Ldr::Token::Name, name),
	                             "object not found:");
	}

      if (local_id)
	{
	  return Ldr::Parser::Val((L4::Server_object*)(local_id & ~3));
	}

      return Ldr::Parser::Val(cap, (err & 1) ? Names::Obj::F_dir : 0);
    }
}

