/*
 * (c) 2008-2009 Technische Universität Dresden
 * This file is part of TUD:OS and distributed under the terms of the
 * GNU General Public License 2.
 * Please see the COPYING-GPL-2 file for details.
 */
#include "script_invoke.h"

#include <l4/re/service>
#include <l4/re/util/name_space_svr>

#include <cstring>

#include "globals.h"
#include "debug.h"
#include "server_obj.h"

static
Ldr::Parser::Val
open_remote_service(Ldr::Scanner::Context const &ctxt, L4::Cap<void> svc,
                    int argc, cxx::String const *argv)
{
  (void)argc;
  L4::Cap<L4::Kobject> new_cap = object_pool.cap_alloc()->alloc<L4::Kobject>();
  int err = L4::cap_cast<L4Re::Service>(svc)->open(new_cap, argv[0].start(),
                                                   argv[0].len());
  if (err < 0)
    {
      Err().printf("%s:%d: invocation of service failed: '%s'\n",
	  ctxt.file(), ctxt.line(), l4sys_errtostr(err));
      object_pool.cap_alloc()->free(new_cap);
      throw Ldr::Parse_exception(ctxt, "");
    }

  return Ldr::Parser::Val(new_cap, L4Re::Util::Names::Obj::F_rw);
}

static
Ldr::Parser::Val
open_local_service(Ldr::Scanner::Context const &ctxt, L4::Server_object *svc,
                   int argc, cxx::String const *argv)
{
  Moe::Local_service *lsvc = dynamic_cast<Moe::Local_service *>(svc);
  if (!lsvc)
    throw Ldr::Parse_exception(ctxt, "invocation not supported");

  return Ldr::Parser::Val(lsvc->open(argc, argv));
}

Ldr::Parser::Val
Moe_obj_mgr::invoke(Ldr::Parser::Val const &obj, int argc,
                    cxx::String const *argv,
                    Ldr::Scanner::Context const &ctxt) const
{
  switch (obj.type())
    {
    case Ldr::Parser::Val::Cap:
      return open_remote_service(ctxt, obj.cap(), argc, argv);

    case Ldr::Parser::Val::Local_entry:
      if (!obj.entry()->obj()->is_trusted())
	throw Ldr::Parse_exception(ctxt, "cannot call untrusted object");

      if(obj.entry()->obj()->is_local())
	return open_local_service(ctxt, obj.entry()->obj()->obj(), argc, argv);
      else
	return open_remote_service(ctxt, obj.entry()->obj()->cap(), argc, argv);

    case Ldr::Parser::Val::Server_object:
      return open_local_service(ctxt, obj.svr_obj(), argc, argv);

    default:
      throw Ldr::Parse_exception(ctxt, "invocation not supported");
    }
}

